package org.ow2.util.ee.metadata.ejbjar.impl.configurator;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarDeployableMetadata;
import org.ow2.util.scan.api.configurator.IArchiveConfigurator;
import org.ow2.util.scan.api.configurator.IClassConfigurator;
import org.ow2.util.scan.api.metadata.structures.JClass;

/**
 * EjbJarArchiveMetadataConfigurator.
 * @param <E> deployable type
 * @author Gael Lalire
 */
public class EjbJarDeployableMetadataConfigurator<E extends EJBDeployable<E>> implements IArchiveConfigurator {

    /**
     * archive metadata.
     */
    private IEjbJarDeployableMetadata<E> ejbJarArchiveMetadata;

    /**
     * annotationParsingDesactived.
     */
    private boolean annotationParsingDesactived;

    /**
     * Constructor.
     * @param ejbDeployable the deployable
     */
    public EjbJarDeployableMetadataConfigurator(final E ejbDeployable) {
        this.ejbJarArchiveMetadata = new EjbJarDeployableMetadata<E>(ejbDeployable);
        this.annotationParsingDesactived = false;
    }

    /**
     * @param jClass signature
     * @return class configurator
     */
    public IClassConfigurator createClassConfigurator(final JClass jClass) {
        return new EjbJarClassMetadataConfigurator<E>(jClass, this.ejbJarArchiveMetadata, this.annotationParsingDesactived);
    }

    /**
     * @return the archive metadata
     */
    public IEjbJarDeployableMetadata<E> getEjbJarArchiveMetadata() {
        return this.ejbJarArchiveMetadata;
    }

    /**
     * @return true if annotation parsing is desactived
     */
    public boolean isAnnotationParsingDesactived() {
        return this.annotationParsingDesactived;
    }

    /**
     * Set annotationParsingDesactived.
     * @param annotationParsingDesactived true if you want to desactive annotation parsing
     */
    public void setAnnotationParsingDesactived(final boolean annotationParsingDesactived) {
        this.annotationParsingDesactived = annotationParsingDesactived;
    }

}
