package org.ow2.util.ee.metadata.ejbjar.impl.configurator;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.impl.configurator.CommonMethodMetadataConfigurator;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxAnnotationSecurityPermitAllVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxAnnotationSecurityRolesAllowedVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxEjbTransactionAttributeVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxInterceptorExcludeDefaultInterceptorsVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxInterceptorInterceptorsVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxAnnotationPostConstructVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxAnnotationPreDestroyVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxAnnotationSecurityDenyAllVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbInitVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbPostActivateVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbPrePassivateVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbRemoveVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxEjbTimeoutVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxInterceptorAroundInvokeVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method.JavaxInterceptorExcludeClassInterceptorsVisitor;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * EjbJarMethodMetadataConfigurator.
 * @param <E> deployable type
 * @author Gael Lalire
 */
public class EjbJarMethodMetadataConfigurator<E extends EJBDeployable<E>> extends
        CommonMethodMetadataConfigurator<EjbJarClassMetadata<E>, EjbJarMethodMetadata<E>, EjbJarFieldMetadata<E>> {

    /**
     * Constructor.
     * @param jMethod signature
     * @param classMetadata parent
     * @param annotationParsingDesactived annotationParsingDesactived
     */
    public EjbJarMethodMetadataConfigurator(final JMethod jMethod, final EjbJarClassMetadata<E> classMetadata,
            final boolean annotationParsingDesactived) {
        super(new EjbJarMethodMetadata<E>(jMethod, classMetadata));

        if (!annotationParsingDesactived) {
            registerAnnotationVisitor();
        }

    }


    /**
     * register annotation visitor.
     */
    @Override
    protected void registerAnnotationVisitor() {

        super.registerAnnotationVisitor();

        EjbJarMethodMetadata<E> ejbJarMethodMetadata = getMethodMetadata();

        // add @Init
        getAnnotationVisitors().put(JavaxEjbInitVisitor.TYPE, new JavaxEjbInitVisitor<E>(ejbJarMethodMetadata));

        // add @Remove
        getAnnotationVisitors().put(JavaxEjbRemoveVisitor.TYPE, new JavaxEjbRemoveVisitor<E>(ejbJarMethodMetadata));

        // add @TransactionAttribute
        getAnnotationVisitors().put(JavaxEjbTransactionAttributeVisitor.TYPE,
                new JavaxEjbTransactionAttributeVisitor<EjbJarMethodMetadata<E>>(ejbJarMethodMetadata));

        // add @PostConstruct
        getAnnotationVisitors().put(JavaxAnnotationPostConstructVisitor.TYPE,
                new JavaxAnnotationPostConstructVisitor<E>(ejbJarMethodMetadata));
        // add @PreDestroy
        getAnnotationVisitors().put(JavaxAnnotationPreDestroyVisitor.TYPE,
                new JavaxAnnotationPreDestroyVisitor<E>(ejbJarMethodMetadata));
        // add @PostActivate
        getAnnotationVisitors().put(JavaxEjbPostActivateVisitor.TYPE, new JavaxEjbPostActivateVisitor<E>(ejbJarMethodMetadata));
        // add @PrePassivate
        getAnnotationVisitors().put(JavaxEjbPrePassivateVisitor.TYPE, new JavaxEjbPrePassivateVisitor<E>(ejbJarMethodMetadata));

        // add @Timeout
        getAnnotationVisitors().put(JavaxEjbTimeoutVisitor.TYPE, new JavaxEjbTimeoutVisitor<E>(ejbJarMethodMetadata));

        // add @AroundInvoke
        getAnnotationVisitors().put(JavaxInterceptorAroundInvokeVisitor.TYPE,
                new JavaxInterceptorAroundInvokeVisitor<E>(ejbJarMethodMetadata));

        // add @Interceptors
        getAnnotationVisitors().put(JavaxInterceptorInterceptorsVisitor.TYPE,
                new JavaxInterceptorInterceptorsVisitor<EjbJarMethodMetadata<E>>(ejbJarMethodMetadata));

        // add @ExcludeClassInterceptors
        getAnnotationVisitors().put(JavaxInterceptorExcludeClassInterceptorsVisitor.TYPE,
                new JavaxInterceptorExcludeClassInterceptorsVisitor<E>(ejbJarMethodMetadata));

        // add @RolesAllowed
        getAnnotationVisitors().put(JavaxAnnotationSecurityRolesAllowedVisitor.TYPE,
                new JavaxAnnotationSecurityRolesAllowedVisitor<EjbJarMethodMetadata<E>>(ejbJarMethodMetadata));

        // add @PermitAll
        getAnnotationVisitors().put(JavaxAnnotationSecurityPermitAllVisitor.TYPE,
                new JavaxAnnotationSecurityPermitAllVisitor<EjbJarMethodMetadata<E>>(ejbJarMethodMetadata));

        // add @DenyAll
        getAnnotationVisitors().put(JavaxAnnotationSecurityDenyAllVisitor.TYPE,
                new JavaxAnnotationSecurityDenyAllVisitor<E>(ejbJarMethodMetadata));

        // add @ExcludeDefaultInterceptors
        getAnnotationVisitors().put(JavaxInterceptorExcludeDefaultInterceptorsVisitor.TYPE,
                new JavaxInterceptorExcludeDefaultInterceptorsVisitor<EjbJarMethodMetadata<E>>(ejbJarMethodMetadata));

    }


}
